/* -*-	Mode:C++; c-basic-offset:8; tab-width:8; indent-tabs-mode:t -*- */
/*
 * Copyright (c) 1994 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /nfs/jade/vint/CVSROOT/ns-2/queue/drop-tail.h,v 1.19 2004/10/28 23:35:37 haldar Exp $ (LBL)
 */

#ifndef ns_broadbandqueue_h
#define ns_broadbandqueue_h

#include <string.h>
#include <math.h>
#include "queue.h"
#include "config.h"
#include "bbqueue.h"


class BBQueue;

class BBQueueHandler : public Handler {
public:
	inline BBQueueHandler(BBQueue& q) : queue_(q) {}
	void handle(Event*);
private:
	BBQueue& queue_;
};


class BBQueue : public Queue {
  public:

	BBQueue() : bh_(*this), bq_() { 
		q_ = new PacketQueue; 
		pq_ = q_;

		/* The queue limit parameter is not honoured here! */

		unblock_on_resume_ = true;
		link_is_ready_ = false;

		bind("delayMicros", &bq_.delayMicros);
		bind("capacityPackets", &bq_.capacityPackets);
		bind("capacityBytes", &bq_.capacityBytes);
		bind("lineRateBps", &bq_.lineRateBps);
	}
	~BBQueue() {
		delete q_;
	}

	void handleNextEvent();

  protected:
	void reset();
	int command(int argc, const char*const* argv); 
	void enque(Packet*);
	Packet* deque();

	PacketQueue *q_;	/* underlying FIFO queue */
	BBQueueHandler bh_;     /* Handler for internal events */
	Event intr_;            /* Internal event */

	BroadbandQueue bq_;     /* Broadband queue model */

  private:
	/* Time convert functions to interface with bbqueue.cc */
	long long getTimeMicros(const double now)
	{
		struct timeval tv;
		tv.tv_sec = (long) rint (now);
		tv.tv_usec = (long) rint ((now - tv.tv_sec) * 1000000);
		return tv.tv_sec * 1000000LL + tv.tv_usec;
	}

	double getTimeSecs(const long long now)
	{
		struct timeval tv;
		tv.tv_sec = (long) rint(now / 1000000);
		tv.tv_usec = (long) rint (now - (tv.tv_sec*1000000));
		return tv.tv_sec + (tv.tv_usec/1000000.0);
	}

	bool link_is_ready_; /* This tells us if we can pass a new packet to the link */
};

#endif
