# Create a simulator object
set ns [new Simulator]

# Set trace file name
set traceallfile out.nam
set statfile out.stat

# Remove old trace files
exec rm -f $traceallfile
exec rm -f $statfile

# Set TCP trace file name
set trace_var [open $statfile w]

# Open the NAM trace file and tell ns to put traces in the file
set allchan [open $traceallfile w]
$ns namtrace-all $allchan

# Set up three agents; The converter will sit in between src and dst
set src [new Agent/TCP/Newreno]
set dst [new Agent/TCPSink]

# Set up three nodes, one for each agent
set node0 [$ns node]
set node1 [$ns node]


#######################################################
# Insert the broadband model
#######################################################
#
# In BB networks, up and downstream are different

# DOWNSTREAM: 4Mbps, queue allows 120ms qdelay, Cable-like propagation delays
Queue/BBQueue set delayMicros 5
Queue/BBQueue set capacityPackets -1
Queue/BBQueue set capacityBytes 60000
Queue/BBQueue set lineRateBps 4000000
# ns2 is doing the delay, link bandwidth must be higher than
# the one we are using in the BBQueue!
$ns simplex-link $node0 $node1 10Mb 25ms BBQueue

# UPSTREAM: 220kbps
Queue/BBQueue set delayMicros 0
Queue/BBQueue set capacityPackets -1
Queue/BBQueue set capacityBytes 60000
Queue/BBQueue set lineRateBps 220000
$ns simplex-link $node1 $node0 10Mb 25ms BBQueue

#######################################################


# Attach agents to respective nodes
$ns attach-agent $node0 $src
$ns attach-agent $node1 $dst

# Connect sender to receiver and vise versa
$ns simplex-connect $src $dst
$ns simplex-connect $dst $src

# Create FTP application and attach it to the tcp agent
set ftp1 [new Application/FTP]
$ftp1 attach-agent $src

# TCP Parameters
$src set packetSize_ 1460
$src set maxcwnd_ 1000
$src set window_ 1000
$src set tcpTick_ 0.001

# Queue sizes in packets
# The parameter queue-limit is ignored for the BBQueue!
# $ns queue-limit $node0 $node1 1000
# $ns queue-limit $node1 $node0 1000

# Print all on a single trace event
$src set trace_all_ 1
# Print all variables in one line, update is everytime a variable changes
$src set trace_all_oneline_ 1

# Attach the trace file to the TCP Source
$src attach $trace_var

# Trace these variables
$src trace cwnd_
$src trace ssthresh_
$src trace ack_
$src trace dupacks_
$src trace rtt_
$src trace srtt_
$src trace maxseq_
$src trace highest_ack_
$src trace seqno_
$src trace rttvar_
$src trace backoff_
$src trace tcp_tick_

# We call this procedure in the end to generate the trace
proc finish {} {
    global ns allchan
    $ns flush-trace
    close $allchan
#    exec ../nam-1.11/nam out.nam &
    exit 0
}

$ns at 0.1 "$ftp1 start"
$ns at 10.1 "$ftp1 stop"

# Finish the simulation when TCP is done with the transfer
$src proc done {} {"finish"}

$ns run
